import { Request, Response } from "express";
import MissionService from "../../../service/operation/missionService";
import MissionFactory from "../../../factory/admin/operation/mission";
import { DeleteMissionCommand, UpdateMissionCommand } from "../../../command/operation/mission/missionCommand";
import MissionCommandHandler from "../../../command/operation/mission/missionCommandHandler";
import SocketServer from "../../../websocket";

/**
 * @description get all missions
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllMissions(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let noLimit = req.query.noLimit === "true";

  let [missions, total] = await MissionService.getAll({ offset, count, noLimit });

  res.json({
    missions: MissionFactory.mapToBaseShort(missions),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get mission by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getMissionById(req: Request, res: Response): Promise<any> {
  const missionId = req.params.id;
  let mission = await MissionService.getById(missionId);

  res.json(MissionFactory.mapToSingleShort(mission));
}

/**
 * @description create mission
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createMission(req: Request, res: Response): Promise<any> {
  let missionId = await MissionCommandHandler.create();

  SocketServer.broadcastNewMission(missionId);

  res.status(200).send(missionId);
}

/**
 * @description update mission by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateMissionById(req: Request, res: Response): Promise<any> {
  const missionId = req.params.id;
  const title = req.body.code;
  const keyword = req.body.type || null;

  let updateMission: UpdateMissionCommand = {
    id: missionId,
    title,
    keyword,
  };
  await MissionCommandHandler.update(updateMission);

  res.sendStatus(204);
}

/**
 * @description delete mission by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteMissionById(req: Request, res: Response): Promise<any> {
  const missionId = req.params.id;

  let deleteMission: DeleteMissionCommand = {
    id: missionId,
  };
  await MissionCommandHandler.delete(deleteMission);

  res.sendStatus(204);
}
